package com.test.couchbase;


import com.couchbase.client.java.error.DocumentDoesNotExistException;
import com.test.couchbase.util.CouchbaseUtils;
import io.vertx.core.AbstractVerticle;
import io.vertx.core.http.HttpHeaders;
import io.vertx.core.http.HttpMethod;
import io.vertx.core.http.HttpServer;
import io.vertx.core.http.HttpServerOptions;
import io.vertx.core.json.JsonObject;
import io.vertx.ext.web.Router;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.BodyHandler;
import io.vertx.ext.web.handler.LoggerFormat;
import io.vertx.ext.web.handler.LoggerHandler;
import io.vertx.ext.web.handler.ResponseTimeHandler;
import io.vertx.rx.java.RxHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.exception.ExceptionUtils;
import rx.Observable;
import rx.Scheduler;

import java.util.NoSuchElementException;
import java.util.concurrent.TimeUnit;

@Slf4j
public class ServerVerticle extends AbstractVerticle {

    @Override
    public void start() throws Exception {
        super.start();
        Router router = Router.router(vertx);
        //router.route().handler(LoggerHandler.create(LoggerFormat.SHORT));
        router.route().handler(ResponseTimeHandler.create());
        router.post().handler(BodyHandler.create());
        router.put().handler(BodyHandler.create());
        router.patch().handler(BodyHandler.create());
        router.delete().handler(BodyHandler.create());
        HttpServer server = vertx.createHttpServer(
                new HttpServerOptions()
                        .setTcpFastOpen(true)
                        .setTcpNoDelay(true)
                        .setTcpQuickAck(true)
                        .setTcpCork(false)
                        .setReusePort(true)
                        .setReuseAddress(true)
                        .setTcpKeepAlive(true)
                        .setCompressionSupported(true));
        router.route(HttpMethod.GET, "/healthz").handler(this::getHealth);
        router.route(HttpMethod.GET, "/getDetails/:id").handler(this::getDetails);
        server.requestHandler(router).listen(8080, handler -> {
            if (handler.succeeded()) {
                log.info("{}", "Server started om port 8080");
            } else {
                handler.cause().printStackTrace();
            }
        });
    }


    private void getHealth(RoutingContext routingContext) {
        routingContext
                .response()
                .putHeader(HttpHeaders.CONTENT_TYPE, "application/json")
                .end(new JsonObject().put("success", "true").toString());
    }

    private void getDetails(RoutingContext routingContext) {
        Scheduler scheduler = RxHelper.scheduler(vertx.getOrCreateContext());
        final StringBuffer keyToFetch = new StringBuffer();
        keyToFetch.append("ARNG_CMP_ARR::").append(routingContext.pathParam("id").trim()).append("::V1");
        final long startTime = System.currentTimeMillis();
        Observable
                .just(keyToFetch.toString())
                .flatMap(key -> CouchbaseUtils.INSTANCE.getAsyncBucket().get(key))
                .doOnNext(t -> log.info("couchbaseTimeTaken:{}", System.currentTimeMillis() - startTime))
                .subscribeOn(scheduler)
                .observeOn(scheduler)
                .subscribe(
                        result -> {
                            routingContext
                                    .response()
                                    .putHeader(HttpHeaders.CONTENT_TYPE, "application/json")
                                    .end(result.content().toString());
                        },
                        error -> {
                            if (error instanceof NoSuchElementException || error instanceof DocumentDoesNotExistException) {
                                routingContext
                                        .response()
                                        .setStatusCode(404)
                                        .putHeader(HttpHeaders.CONTENT_TYPE, "application/json")
                                        .end(new JsonObject().put("error", ExceptionUtils.getStackTrace(error)).toString());

                            } else {
                                routingContext
                                        .response()
                                        .setStatusCode(500)
                                        .putHeader(HttpHeaders.CONTENT_TYPE, "application/json")
                                        .end(new JsonObject().put("error", ExceptionUtils.getStackTrace(error)).toString());
                            }
                        }
                );
    }


    @Override
    public void stop() throws Exception {
        super.stop();
        try {
            CouchbaseUtils.INSTANCE.getCluster().disconnect(2, TimeUnit.MINUTES);
        } finally {
            CouchbaseUtils.INSTANCE.getCouchbaseEnvironment().shutdown(2, TimeUnit.MINUTES);
        }
    }
}
