package com.test.couchbase.util;

import com.couchbase.client.core.env.DefaultCoreEnvironment;
import com.couchbase.client.core.env.resources.IoPoolShutdownHook;
import com.couchbase.client.deps.io.netty.channel.EventLoopGroup;
import com.couchbase.client.deps.io.netty.channel.epoll.Epoll;
import com.couchbase.client.deps.io.netty.channel.epoll.EpollEventLoopGroup;
import com.couchbase.client.deps.io.netty.channel.nio.NioEventLoopGroup;
import com.couchbase.client.deps.io.netty.util.concurrent.DefaultThreadFactory;
import com.couchbase.client.java.AsyncBucket;
import com.couchbase.client.java.Bucket;
import com.couchbase.client.java.Cluster;
import com.couchbase.client.java.CouchbaseCluster;
import com.couchbase.client.java.env.CouchbaseEnvironment;
import com.couchbase.client.java.env.DefaultCouchbaseEnvironment;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.time.Duration;
import java.util.concurrent.ThreadFactory;

@Slf4j
public enum CouchbaseUtils {
    INSTANCE();
    @Getter
    private Cluster cluster;
    private Bucket bucket;
    @Getter
    private AsyncBucket asyncBucket;
    @Getter
    private CouchbaseEnvironment couchbaseEnvironment;

    private CouchbaseUtils() {
        if (cluster == null) {
            System.out.println("Couchbase native transport: " + Epoll.isAvailable());
            ThreadFactory ioPoolThreadFactory = new DefaultThreadFactory("cb-io", true);
            EventLoopGroup ioPoolEventLoop = Epoll.isAvailable() ? new EpollEventLoopGroup(DefaultCoreEnvironment.IO_POOL_SIZE, ioPoolThreadFactory) : new NioEventLoopGroup(DefaultCoreEnvironment.IO_POOL_SIZE);
            ThreadFactory kvIoPoolThreadFactory = new DefaultThreadFactory("cb-kv-io", true);
            EventLoopGroup kvIoPoolEventLoop = Epoll.isAvailable() ? new EpollEventLoopGroup(DefaultCoreEnvironment.KEYVALUE_ENDPOINTS > 3 ? DefaultCoreEnvironment.KEYVALUE_ENDPOINTS : 3, kvIoPoolThreadFactory) : new NioEventLoopGroup(DefaultCoreEnvironment.KEYVALUE_ENDPOINTS > 3 ? DefaultCoreEnvironment.KEYVALUE_ENDPOINTS : 3, kvIoPoolThreadFactory);
            couchbaseEnvironment = DefaultCouchbaseEnvironment
                    .builder()
                    .connectTimeout(Duration.ofMinutes(2).toMillis())
                    //.autoreleaseAfter(Duration.ofMinutes(2).toMillis())
                    .ioPool(ioPoolEventLoop, new IoPoolShutdownHook(ioPoolEventLoop))
                    .kvIoPool(kvIoPoolEventLoop, new IoPoolShutdownHook(kvIoPoolEventLoop))
                    .build();
            cluster = CouchbaseCluster.create(couchbaseEnvironment, "<Host>");
            bucket = cluster.openBucket("<Bucket>", "<Password>");
            asyncBucket = bucket.async();
        }
    }
}
